<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Import_master extends CI_Controller
{
	public function __construct()
	{
		parent::__construct();

		// INI settings
		ini_set('max_execution_time', 0);
		ini_set('memory_limit', '-1');
		ini_set("pcre.backtrack_limit", 23001337);
		ini_set("pcre.recursion_limit", 23001337);

		// Set your modules here which you want to set import for
		$this->import_modules = array(
			'BULK_IMPORT_EMPLOYEE_WORKING_HOURS' => array(
				'process' 		=> 'process_employee_working_hours',
				'header_map' 	=> array(
					'User Name'		=> 'user_name',
					'Working Date' 	=> 'date',
					'Department'	=> 'department',
					'Task Type' 	=> 'task_type',
					//'Hours'			=> 'hours',
					'Start Time'	=> 'start_time',
					'End Time'		=> 'end_time'
				)
			),
			'IMPORT_ITEM_TO_LIST' => array(
				'process' 		=> 'process_import_item_to_list',
				'header_map' 	=> array(
					'Store Name'    => 'store_name',
					'SKU'           => 'sku',
					'UPC'           => 'upc',
					'Selling Price' => 'selling_price',
					'Floor Price'   => 'floor_price',
					'Ceiling Price' => 'ceiling_price'
				)
			)
		);

		// Loading libraries and all other required stuffs
		$this->load->library('excel/excel');
		$this->load->helper('file');
		$this->load->helper('array_helper');
		$this->load->helper('download');
		$this->load->model('import/import_master_model');

		// Initializing variables
		$this->filter 			= array();
		$this->header_map 		= array();
		$this->Error_Array 		= array();
		$this->Error_Array_Multi_Sheet = array();
		$this->CurrentActiveSheetName = NULL;
		$this->folder_path		= 'bulk_import_excel/';
		$this->bulk_excel_path 	= $this->config->item("upload_path").$this->folder_path;
		$this->Error_File_Path = $this->bulk_excel_path;
	}


	/*
	@Description 	: Process bulk import of employee working hours
	@Author 		: Akshit Arora
	@Input 			: excel data
	@Output 		: 
	@Date 			: 27-02-2019
	*/
	public function process_employee_working_hours($import_data)
	{
		if(count($import_data))
		{
			//pr($import_data);
			$data_to_insert = array();

			$this->load->model('report/employee_working_hrs_report_model');

			$task_types = array_unique(array_column($import_data, 'task_type'));
			$user_names = array_unique(array_column($import_data, 'user_name'));

			// $valid_task_types = $this->import_master_model->get_task_types($task_types);
			// $valid_task_types = helper_array_column($valid_task_types, 'task_name', 'id');

			$valid_users = $this->import_master_model->get_users($user_names);
			$valid_users = helper_array_column($valid_users, 'user_name', 'id');

			foreach($import_data as $log_hours)
			{
				$error_log = array();
				
				// Validations
				// if($log_hours['hours'] > 24)
				// {
				// 	$error_log[] = 'Working hours cannot exceed to 24';
				// }

				// if(!is_numeric($log_hours['hours']))
				// {
				// 	$error_log[] = "Working hours value should be numeric";
				// }

                if($log_hours['start_time'] == '')
				{
					$error_log[] = 'Please enter start time';
				}

				 if($log_hours['end_time'] == '')
				{
					$error_log[] = 'Please enter end time';
				}


				// Checking date
				$date = $log_hours['date'];
				$date = explode('-', $date);

				if(!checkdate($date[1], $date[2], $date[0]))
				{
					$error_log[] = "Please enter a valid date in the format YYYY-MM-DD";
				}

				if(!get_value($valid_users, $log_hours['user_name'], false))
				{
					$error_log[] = 'User Name not found';
				}

				$valid_departments = array();

				if(get_value($valid_users, $log_hours['user_name'], false))
				{
					$valid_departments = $this->employee_working_hrs_report_model->get_employee_departments($valid_users[$log_hours['user_name']]);

					$valid_departments = helper_array_column($valid_departments, 'name', 'id');

					if(!get_value($valid_departments, $log_hours['department'], false))
					{
						$error_log[] = 'Invalid Department';
					}
				}

				if(get_value($valid_departments, $log_hours['department'], false))
				{
					$valid_task_types = $this->import_master_model->get_task_types(array(), get_value($valid_departments, $log_hours['department']));
					$valid_task_types = helper_array_column($valid_task_types, 'task_name', 'id');

					if(isset($log_hours['task_type']) && !get_value($valid_task_types, $log_hours['task_type'], false))
					{
						$error_log[] = 'Task type is not valid';
					}


				}

				if(count($error_log))
				{
					$this->set_row_error($error_log, $log_hours);
					continue;
				}

				if($log_hours['end_time']!="")
				{
					$start_hour =  strtotime($log_hours['end_time']) - strtotime($log_hours['start_time']);
					$working_hours = gmdate("H:i", $start_hour); 
				}
				else
				{
					$log_hours['end_time'] = NULL;
					$working_hours = 0;
				}

				// Enter value
				$data_to_insert[] = array(
					'user_id' 		=> get_value($valid_users, $log_hours['user_name']),
					'task_type_id' 	=> get_value($valid_task_types, $log_hours['task_type']),
					'department_id'	=> get_value($valid_departments, $log_hours['department']),
					'date'			=> $log_hours['date'],
					'hours'			=> $working_hours,
					'start_time'	=> $log_hours['start_time'],
					'end_time'	    => $log_hours['end_time'],
					'inserted_by'	=> $this->log_in_user_id,
					'inserted_date'	=> get_inserted_date_time()
				);
			}

			if(count($data_to_insert))
			{
				$this->common_model->insert_batch('employee_working_hrs', $data_to_insert);
			}
		}
	}

	/*
    @Description 	: Import item to list
    @Author 		: Harshad Hirapara
    @Input 			: module name
    @Output 		:
    @Date 			: 03-07-2019
    */
    public function process_import_item_to_list($import_data=array())
    {	
    	if(count($import_data))
    	{
    		$sku_array = array_unique(array_column($import_data, 'sku'));

    		$product_list = $this->import_master_model->get_product_data($sku_array);

    		$store_list = $this->import_master_model->get_store_list();

    		$insert_array =array();

    		foreach($import_data as $key => $product)
    		{	
    			$error_log = array();
    			$sku = trim($product['sku']);
    			$store_name = trim($product['store_name']);

    			$store_id = isset($store_list[$store_name]) ? $store_list[$store_name] :NULL;

    			if(isset($product_list[$sku]))
    			{	
    				$existing_data = $product_list[$sku];

    				if($store_id!="")
    				{
    					$insert_array[] = array(
    						'product_id'    => $existing_data['id'],
    						'store_id'      => $store_id,
    						'title'         => $existing_data['title'],
    						'upc'           => get_value($product,'upc')!="" ?get_value($product,'upc') : $existing_data['upc'],
    						'selling_price' => get_value($product,'selling_price')!="" ?get_value($product,'selling_price') : $existing_data['selling_price'],
    						'floor_price'   => get_value($product,'floor_price')!="" ?get_value($product,'floor_price') : $existing_data['floor_price'],
    						'ceiling_price' => get_value($product,'ceiling_price')!="" ?get_value($product,'ceiling_price') : $existing_data['ceiling_price'],
    						'inserted_by' => $this->log_in_user_id,
    						'inserted_date' => get_inserted_date_time(),
    					);
    				}
    				else
    				{
    					$error_log[]='Please enter valid store name';

    					$this->set_row_error($error_log, $product);
    				}

    			}
    			else
    			{	
    				$error_log[]='Product not found in system';

    				$this->set_row_error($error_log, $product);
    			}
    		}

    		if(count($insert_array)>0)
    		{
    			$this->common_model->insert_batch('item_to_list',$insert_array);
    		}
    	}
    }

	// TODO : Make upload function as well.

	/*
    @Description 	: Starting point of import cron
    @Author 		: Akshit Arora
    @Input 			: module name
    @Output 		:
    @Date 			: 27-02-2019
    */
    public function start_process($module_name, $debug_mode = false)
    {
		// Check if the module is defined in the constructor
    	if(array_key_exists($module_name, $this->import_modules))
    	{
			// Get the scheduler details
    		$scheduler_details = $this->import_master_model->get_scheduler_details($module_name);
    		
			// Check if we have the details
    		if(!empty($scheduler_details))
    		{
    			try
    			{
					// Set header map
    				$this->header_map = get_value($this->import_modules, $module_name.'|header_map', false);

    				if(!$this->header_map)
    				{
    					echo "Error: Headers not defined";
    					exit;
    				}

					// Set calling function
    				$calling_function = get_value($this->import_modules, $module_name.'|process', false);

    				if(!$calling_function || !method_exists($this, $calling_function))
    				{
    					echo "Error: Process Function not defined";
    					exit;
    				}

					// If headers and function to process found, now start the process
    				if(!$debug_mode)
    				{
    					$this->import_master_model->set_process_status($scheduler_details['id'], 'InProcess');
    				}

					// Set the logged in user id as who added the file
    				$this->log_in_user_id = $scheduler_details["inserted_by"];

					// Get the file data
    				$file_data = $this->get_import_file_data($scheduler_details, $debug_mode);

					// If we got any data from the file
    				if(count($file_data))
    				{
						// Call the set function
    					$this->$calling_function($file_data);
    				}
    				else
    				{
    					$Error_Msg = 'No data found in file';

    					if(!$debug_mode)
    					{
    						$this->import_master_model->set_process_status($scheduler_details['id'], 'Completed', array('error' => $Error_Msg));
    					}

    					exit;
    				}

    				if(!empty($this->Error_Array))
    				{
    					$this->Error_Array_Multi_Sheet[$this->CurrentActiveSheetName] = $this->Error_Array;
    					$this->Error_Array = array();

    					$error_file_name = $this->writeErrorIntoExcel();

    					if(!$debug_mode)
    					{
    						$this->import_master_model->set_process_status($scheduler_details['id'], '', array( "error_file_name" => $error_file_name));
    					}
    				}

    				$Count_End = count($file_data);

    				if(!$debug_mode)
    				{
    					$this->import_master_model->set_process_status($scheduler_details['id'], 'Completed', array('processed_count' => $Count_End));
    				}
    			}
    			catch(Exception $e)
    			{
    				$error_msg = (string)$e->getMessage();

    				if(!$debug_mode)
    				{
    					$this->import_master_model->set_process_status($scheduler_details['id'], 'Completed', array('error' => serialize(array('error'=>$error_msg))));
    				}
    			}
    		}
    		else
    		{
    			echo "No Pending Processes";
    		}
    	}
    	else
    	{
    		echo "Error: Import details not defined";
    		exit;
    	}
    }

	/*
    @Description 	: Function to get the excel file data
    @Author 		: 
    @Input 			:
    @Output 		:
    @Date 			: 27-02-2019
    */
    private function get_import_file_data($scheduler_details, $debug_mode)
    {
    	$scheduler_details["filter"] 	= unserialize($scheduler_details["filter"]);
    	$this->filter 					= $scheduler_details["filter"];
    	$file_full_path 				= $this->bulk_excel_path.$scheduler_details["filter"]["file_name"];
    	$excel_file_data 				= array();

    	if(file_exists($file_full_path)==false)
    	{	
    		echo $Error_Msg = "File not In System";
    		if(!$debug_mode)
    		{
    			$this->import_master_model->set_process_status($scheduler_details['id'], 'Completed', array('error' => $Error_Msg));
    		}

    		exit;
    	}

    	$this->CurrentFileData_DB 	= $scheduler_details;
    	$this->current_file_type 	= $FileType = $scheduler_details["filter"]["file_type"];
    	$FileType 					= strtolower($this->current_file_type)=='csv'?'CSV':'Excel2007';
    	$this->CurrentFile  		= trim($scheduler_details["filter"]["file_name"]);
    	$module_type 				= isset($scheduler_details["filter"]["module_type"])?$scheduler_details["filter"]["module_type"]:NULL;

    	$this->readExcel($this->CurrentFile,$FileType);

    	$Sheet_Index = 0;

    	$excel_Data = array();

    	$this->CurrentPHPExcel_Object->setActiveSheetIndex($Sheet_Index);

    	$this->CurrentActiveSheet 	= $this->CurrentPHPExcel_Object->getActiveSheet();
    	$this->CurrentActiveSheetName 	= $this->CurrentActiveSheet->getTitle();
    	$maxCol 					= $this->CurrentActiveSheet->getHighestColumn();
    	$maxRow 					= $this->CurrentActiveSheet->getHighestRow();
    	$rangeHeaderString 			= 'A1:'.$maxCol."1";

    	$HeaderData 				= $this->CurrentActiveSheet->toArray(null,true,true,false,$rangeHeaderString);

    	if($HeaderData)
    	{
    		$this->Header_Array 		= $HeaderData["0"];
    		$this->Header_Array_Orignal = $this->Header_Array = array_trim($this->Header_Array);
    	}

    	$rangeString 						= "";
    	$maxRowTake 						= null;
    	$this->TotalRecord 					= $maxRow-1;

    	$rangeString = "A2:".$maxCol.($maxRow);


    	if($scheduler_details["total_count"]=="0" || $scheduler_details["total_count"]=="")
    	{
    		if(!$debug_mode)
    		{
    			$this->import_master_model->set_process_status($scheduler_details['id'], '', array('total_count' => $this->TotalRecord));
    		}

    		$scheduler_details["total_count"] = $this->TotalRecord;
    	}

    	if($this->CurrentActiveSheet && !empty($this->Header_Array))
    	{
    		$excel_Data = $this->CurrentActiveSheet->toArray(null,true,true,false,$rangeString);

    		$this->Error_Array = array();

    		$excel_file_data = array();

    		foreach($excel_Data as $data)
    		{
    			$excel_row_data = array();

    			foreach($data as $key => $row_value)
    			{
    				$row_key = get_value($this->header_map, @$this->Header_Array[$key], strtolower(@$this->Header_Array[$key]));

    				$excel_row_data[$row_key] = $row_value;
    			}

    			$excel_file_data[] = $excel_row_data;
    		}
    	}

    	return $excel_file_data;
    }

    private function get_filter()
    {
    	return $this->filter;
    }

	/*
	@Description : Function to read excel data
	@Author      : Yogesh Hotchandani
	@Input       :
	@Output      :
	@Date        : 12-05-2017
	*/
	private function readExcel($FileName,$inputFileType="Excel2007",$SheetIndex="0")
	{
		$FileFullPath = $this->bulk_excel_path.$FileName;
		try
		{
			$objReader = PHPExcel_IOFactory::createReader($inputFileType);
			$objReader->setReadDataOnly(true);
			$this->CurrentPHPExcel_Object = PHPExcel_IOFactory::load($FileFullPath);
		}
		catch(Exception $e)
		{
			throw new Exception($e->getMessage(), 1);
		}
	}

	/*
	@Description : Function to write error into excel
	@Author      : Yogesh Hotchandani
	@Input       :
	@Output      :
	@Date        : 12-05-2017
	*/
	public function writeErrorIntoExcel()
	{ 
		try
		{   
			if(!empty($this->Error_Array_Multi_Sheet))
			{
				if($this->CurrentFileData_DB)
				{ 
					$FileName  = $this->CurrentFileData_DB["error_file_name"];
					if($FileName!="")
					{
						$FileName = basename($FileName);
					}
					$FileFullPath = $this->Error_File_Path.$FileName;
					$inputFileType = $this->current_file_type ? $this->current_file_type : 'Excel2007';

					if($inputFileType == 'csv')
					{
						$inputFileType = 'CSV';
					}

					$file_extension = strtolower($inputFileType)=='csv'?'.csv':'.xlsx';
					
					if($this->CurrentFileData_DB["error_file_name"]=="")
					{   
						$objPHPExcel = new PHPExcel();
						$i = 0;
						foreach ($this->Error_Array_Multi_Sheet as $key => $value)
						{   
							if($i==0)
							{
								$objPHPExcel->setActiveSheetIndex($i);
								$objPHPExcel->getActiveSheet()->setTitle($key);

							}
							else
							{
								$objWorkSheet = $objPHPExcel->createSheet($i);
								$objWorkSheet->setTitle($key);

							}
							$i++;
						}
						$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel,$inputFileType);
						$error_file_name = uniqid();
						$FileName = $error_file_name.$file_extension;
						$FileFullPath = $this->Error_File_Path.$FileName;
						$objWriter->save($FileFullPath);
						$objPHPExcel->disconnectWorksheets();

						unset($objPHPExcel,$objWriter);
					}

					$objReader          = PHPExcel_IOFactory::createReader($inputFileType);
					$CurrentPHPExcel_Object = PHPExcel_IOFactory::load($FileFullPath);
					$Sheet_Names   = $CurrentPHPExcel_Object->getSheetNames();
					foreach ($Sheet_Names as $sheet_key => $value)
					{ 
						if(isset($this->Error_Array_Multi_Sheet[$value]) && !empty($this->Error_Array_Multi_Sheet[$value]))
						{  
							$Errors = $this->Error_Array_Multi_Sheet[$value];
							$CurrentActiveSheet = $Sheet_Names[0];
							$CurrentPHPExcel_Object->setActiveSheetIndex($sheet_key);
							$CurrentActiveSheet = $CurrentPHPExcel_Object->getActiveSheet();
							if($CurrentActiveSheet)
							{
								$Heighst_Row = $CurrentActiveSheet->getHighestRow();
								$start_range = "A2";
								if($Heighst_Row=="1")
								{
									$Header_Array = array("Error"=>"Error")+$this->Header_Array_Orignal;
									$CurrentActiveSheet->fromArray($Header_Array,NULL,"A1");
								}
								else
								{
									if($Heighst_Row)
										$start_range = "A".($Heighst_Row+1);
								}
								$CurrentActiveSheet->fromArray($Errors,NULL,$start_range);
							}
						}
					}

					$objWriter = PHPExcel_IOFactory::createWriter($CurrentPHPExcel_Object,$inputFileType);
					$FileFullPath = $this->Error_File_Path.$FileName;
					$objWriter->save($FileFullPath);
					$CurrentPHPExcel_Object->disconnectWorksheets();
					unset($CurrentPHPExcel_Object,$objWriter);

					return $this->folder_path."$FileName";
				}
			}
		}
		catch(Exception $e){throw new Exception($e->getMessage(), 1);}
	}

	private function set_row_error($errors_array, $data_row)
	{
		$error_detail = implode(', ', $errors_array);

		$data_row = array('error' => $error_detail) + $data_row;

		$this->Error_Array[] 	= $data_row;
	}

}
